from typing import Any

__all__ = ["RoadrunnerBootstrap5Config"]


def __getattr__(name: str) -> Any:
    if name == "RoadrunnerBootstrap5Config":
        from .entrypoint import RoadrunnerBootstrap5Config as _obj

        return _obj
    raise AttributeError(f"module '{__name__}' has no attribute '{name}'")



# -*- coding: utf-8 -*-
from importlib import import_module

from django.template.loader import get_template
from django.test import SimpleTestCase

from ocyan.plugin.roadrunner_bs5.models import RoadrunnerPage


class RoadrunnerBs5SmokeTests(SimpleTestCase):
    def test_app_import(self):
        module = import_module("ocyan.plugin.roadrunner_bs5")
        self.assertTrue(hasattr(module, "RoadrunnerBootstrap5Config"))

    def test_page_model_import(self):
        self.assertEqual(RoadrunnerPage.__name__, "RoadrunnerPage")

    def test_roadrunner_page_template_loads(self):
        template = get_template("roadrunner/roadrunner_page.html")
        self.assertIsNotNone(template)

    def test_wagtail_hooks_imports(self):
        module = import_module("ocyan.plugin.roadrunner_bs5.wagtail_hooks")
        self.assertIsNotNone(module)



# Generated by Django 3.2.13 on 2022-08-23 14:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('roadrunner_bs5', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='roadrunnerpage',
            name='noindex',
            field=models.BooleanField(default=False),
        ),
    ]



from .slideshow import slideshow


def convert_html(obj):
    value = obj["value"]
    if "text" in value and "html" not in value:
        value["html"] = value.pop("text")
    return obj


def convert_carousel(obj):
    return slideshow(
        obj,
        lambda slide: (
            '<p><a href="%(url)s">%(caption)s</a></p>' % slide if slide["url"] else ""
        ),
        "carousel",
    )


def convert_iconslider(obj):
    return slideshow(
        obj,
        lambda slide: (
            '<p><a href="%(url)s">%(name)s</a></p>' % slide
            if slide["url"]
            else "<p>%(name)s</p>" % slide
        ),
        "iconslider",
        items=6,
    )


def convert_product(obj):
    obj["type"] = "Product"
    return obj


def convert_googlemaps(obj):
    obj["value"].pop("coordinates", None)
    obj["type"] = "Maps"
    return obj


def convert_vat(obj):
    obj["type"] = "VAT_entry"
    return obj


def convert_embed(obj):
    obj["type"] = "video"
    return obj



from django.apps import apps
from django.db.models import Q, Value
from django.db.models.functions import Lower, Replace
from django.template import Context, Template

from .utils import uuid


def convert_snippet(obj):
    try:
        Snippet = apps.get_model("roadrunner.block")
    except LookupError:
        raise KeyError("There are no snippets")

    try:
        snippets = Snippet.objects.annotate(
            slug=Lower(Replace("name", Value(" "), Value("-")))
        )
        snip = snippets.get(Q(name=obj["type"]) | Q(slug=obj["type"]))
        template = Template(snip.html)
        html = template.render(Context(obj["value"]))
        return {"type": "html", "value": {"html": html}, "id": uuid()}
    except Snippet.DoesNotExist:
        raise KeyError("this is not a snippet")



# -*- coding: utf-8 -*-
from os.path import abspath, dirname, join

from ocyan.core.utils import merge_dicts
from ocyan.plugin.wagtail.block_plugin import BlockPluginConfig


class RoadrunnerBootstrap5Config(BlockPluginConfig):
    def contribute_to_settings(self, settings):
        return merge_dicts(
            settings,
            {
                "ROADRUNNER_REGISTRY_FUNCTION": "ocyan.plugin.roadrunner_bs5.utils.get_roadrunner_block_choices"
            },
            overwrite=True,
        )

    def contribute_to_installed_apps(self, installed_apps):
        installed_apps.wagtail.extend(("ocyan.plugin.roadrunner_bs5", "rr"))
        installed_apps.wagtail.append("wagtail.contrib.table_block")
        installed_apps.ocyan.register_extension(
            "ocyan.plugin.roadrunner_bs5.bs5_wagtail_oscar_integrations_extension",
            "ocyan.plugin.wagtail_oscar_integration",
        )
        return installed_apps

    def contribute_to_blocks(self, blocks):
        from .blocks import (
            EmptyBlock,
            GoogleMapsBlock,
            ParallaxBlock,
            SectionedFAQBlock,
            SlideShowBlock,
        )

        return blocks + [
            ("Maps", GoogleMapsBlock(), "rr"),
            ("Parallax", ParallaxBlock(), "rr"),
            ("Slideshow", SlideShowBlock(), "rr"),
            ("Empty", EmptyBlock(), "rr"),
            ("FAQ", SectionedFAQBlock(), "rr"),
        ]

    def contribute_to_fixtures(self, fixtures):
        directory = dirname(__file__)
        filename = abspath(join(directory, "fixtures/roadrunner_bs5/homepage.json"))
        fixtures.add("wagtailhome", filename, 2)
        return fixtures



"""
This command will replace roadrunner_bs4 pages with roadrunner_bs5 pages.
"""

import json
from string import Template

from django.apps import apps
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.core.management import call_command
from django.core.management.base import BaseCommand
from django.db import connection, transaction

from wagtail.models import Page

from ocyan.plugin.roadrunner_bs5.convert import main as convert
from ocyan.plugin.roadrunner_bs5.convert.utils import REGISTERED_BLOCK_TYPES
from ocyan.plugin.roadrunner_bs5.models import RoadrunnerPage

RAW_UPDATE = Template("UPDATE $db_table SET $column = %s WHERE $id_column = %s")


class Command(BaseCommand):
    help = __doc__

    def add_arguments(self, parser):
        parser.add_argument(
            "--pagetype",
            action="append",
            default=[],
            help="process the custom page types ait",
        )
        parser.add_argument(
            "--image-key",
            action="append",
            default=["image", "product", "category", "template"],
            help="Key of fields that contain id's of images",
        )
        parser.add_argument(
            "--richtext-key",
            action="append",
            default=[
                "block_text",
                "tab_content",
                "html",
                "caption",
                "card_content",
                "modal_content",
                "panel_content",
                "media_content",
                "well_content",
                "body",
                "content",
                "text",
                "paragraph",
            ],
            help="Key of fields that contain richtext data",
        )
        parser.add_argument("--debug", default=False, action="store_true")

    def handle(self, *args, **options):
        self.stdout.write("Registered block types: %s" % REGISTERED_BLOCK_TYPES)

        if (
            settings.ROADRUNNER_REGISTRY_FUNCTION
            == "ocyan.plugin.roadrunner_bs4.utils.get_roadrunner_block_choices"
        ):
            self.stderr.write(
                "You've got 'ocyan.plugin.roadrunner_bs4.utils.get_roadrunner_block_choices' "
                "configured as the ROADRUNNER_REGISTRY_FUNCTION. Please set the ROADRUNNER_REGISTRY_FUNCTION="
                "'ocyan.plugin.roadrunner_bs5.utils.get_roadrunner_block_choices' in your project settings before runing bs4_to_bs5."
            )
            exit(1)

        # pylint: disable=attribute-defined-outside-init
        self.image_keys = options["image_key"]
        self.richtext_keys = options["richtext_key"]

        try:
            with transaction.atomic():
                self.port_templates()
                for model_name in options["pagetype"]:
                    model = apps.get_model(model_name)
                    self.convert_bs4_fields(model)
                self.handle_replacement()
        except Exception as err:  # pylint: disable=broad-except
            self.stdout.write(
                self.style.ERROR(
                    "Something went wrong while replacing bs4 pages with bs5 pages, so no changes have been commited to the database. Error: %s"
                    % err
                )
            )
            if options["debug"]:
                raise

    def convert_bs4_fields(self, model):
        db_table = model._meta.db_table
        fields = []
        for field in model._meta.get_fields():
            if field.__class__.__name__ == "RoadRunnerField":
                fields.append(field)

        id_column = "id"
        if model._meta.parents:
            (parent_pointer,) = model._meta.parents.values()
            id_column = parent_pointer.column

        for instance in model.objects.only(
            "id", "title", *[field.name for field in fields]
        ):
            for field in fields:
                with connection.cursor() as cursor:
                    data = self.convert_bs4_to_bs5(getattr(instance, field.name))
                    query = RAW_UPDATE.substitute(
                        db_table=db_table, column=field.column, id_column=id_column
                    )
                    cursor.execute(query, [data, instance.id])

    def port_templates(self):
        BS4Template = apps.get_model("roadrunner.Template")

        with connection.cursor() as cursor:
            for template in BS4Template.objects.all():
                cursor.execute(
                    "INSERT INTO rr_template (id, name, content) VALUES (%s, %s, %s)",
                    [
                        template.pk,
                        template.name,
                        self.convert_bs4_to_bs5(template.content),
                    ],
                )

    def handle_replacement(self):
        bs4_base_pages = Page.objects.filter(content_type=self.get_bs4_content_type())
        for bs4_base_page in bs4_base_pages:
            bs4_page = bs4_base_page.specific

            # Here we're inserting a new record into the roadrunner_bs5_roadrunnerpage table
            # with the abstractbasepage_ptr_id set to the one of bs4. We're not using RoadrunnerPage.create()
            # here because this will also create new basepages, which is what we don't want.
            with connection.cursor() as cursor:
                cursor.execute(
                    "INSERT INTO roadrunner_bs5_roadrunnerpage (abstractbasepage_ptr_id, content, noindex) VALUES (%s, %s, FALSE)",
                    [
                        bs4_page.abstractbasepage_ptr_id,
                        self.convert_bs4_to_bs5(bs4_page.content),
                    ],
                )

            # Now on the base type, we'll only have to change the content type to the bs5 content type
            # and now the bs5 page will be read instead!
            bs4_base_page.content_type = self.get_bs5_content_type()
            bs4_base_page.save()

        call_command("fixtree")

    def convert_bs4_to_bs5(self, content):
        try:
            result = convert.bs4_to_bs5(
                content.raw_data,
                image_keys=self.image_keys,
                richtext_keys=self.richtext_keys,
            )
        except:
            self.stderr.write(json.dumps(list(content.raw_data), indent=2))
            raise

        return json.dumps(result, indent=2)

    def get_bs4_content_type(self):
        return ContentType.objects.get(
            app_label="roadrunner_bs4", model="roadrunnerbootstrapfourpage"
        )

    def get_bs5_content_type(self):
        return ContentType.objects.get_for_model(RoadrunnerPage)



from django import forms
from django.utils.functional import cached_property
from django.utils.translation import gettext as _

from wagtail.admin.staticfiles import versioned_static
from wagtail.blocks.field_block import FieldBlockAdapter


class PreviewAdminChooserAdapter(FieldBlockAdapter):
    js_constructor = "ocyan.PreviewAdminChooser"

    def js_args(self, block):
        name, values, meta = super().js_args(block)
        meta["strings"]["EMPTY"] = _("No choice has been made")
        return name, values, meta

    @cached_property
    def media(self):
        return super().media + forms.Media(
            js=[
                versioned_static("wagtailadmin/js/telepath/blocks.js"),
                versioned_static("roadrunner/roadrunner.js"),
                versioned_static("wagtailadmin/js/roadrunner_bs5.js"),
            ],
        )


class PreviewSelectAdapter(PreviewAdminChooserAdapter):
    js_constructor = "ocyan.PreviewSelect"



from .utils import uuid


def slideshow(obj, formatslide_body, classname, items=1):
    return {
        "type": "Slideshow",
        "value": {
            "slides": [
                {
                    "type": "item",
                    "value": {
                        "image": slide["image"],
                        "body": formatslide_body(slide),
                        "position": "middle-left",
                    },
                    "id": uuid(),
                }
                for slide in obj["value"]
            ],
            "items": items,
            "slideby": "1",
            "controls": True,
            "autoplaytimeout": "",
            "loop": True,
            "nav": False,
            "autoheight": True,
            "autoplay": False,
            "fixed_height": "",
            "lazy": False,
            "styling": {
                "margin": "",
                "padding": "",
                "border": "",
                "border_radius": "",
                "background_color": "",
                "color": "",
                "font_size": None,
                "text_align": "",
                "block_classes": classname,
            },
        },
        "id": uuid(),
    }



# Generated by Django 3.2.13 on 2022-06-30 14:47

from django.db import migrations, models
import django.db.models.deletion
import rr.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('ocyan_plugin_wagtail', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='RoadrunnerPage',
            fields=[
                ('abstractbasepage_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='ocyan_plugin_wagtail.abstractbasepage')),
                ('content', rr.fields.RoadRunnerField(blank=True, null=True)),
            ],
            options={
                'verbose_name': 'Pagemaker page',
                'verbose_name_plural': 'Pagemaker pages',
            },
            bases=('ocyan_plugin_wagtail.abstractbasepage',),
        ),
    ]



from django.core.exceptions import ImproperlyConfigured
from django.db import migrations


class RenamePrimaryKey(migrations.RenameField):
    @property
    def reversible(self):
        return False

    def update_primary_key_relations(self, schema_editor, new_field):
        if schema_editor.connection.vendor == "sqlite":
            for rel in new_field.model._meta.related_objects:
                if not rel.many_to_many:
                    schema_editor._remake_table(rel.related_model)  # pylint: disable=protected-access

    def database_forwards(self, app_label, schema_editor, from_state, to_state):
        super().database_forwards(app_label, schema_editor, from_state, to_state)

        to_model = to_state.apps.get_model(app_label, self.model_name)
        new_field = to_model._meta.get_field(self.new_name)

        self.update_primary_key_relations(schema_editor, new_field)

    def database_backwards(self, app_label, schema_editor, from_state, to_state):
        raise NotImplementedError("You cannot reverse this operation")


# pylint: disable=W0613
def disable_check_constraints(apps, schema_editor):
    if schema_editor.connection.vendor == "sqlite":
        schema_editor.connection.check_constraints = lambda *args, **x: None


class Bs4ToBs5BasePageMigration(migrations.Migration):
    new_bases = None
    model_name = None
    old_fk = None
    new_fk = None
    new_field = None

    def apply(self, project_state, schema_editor, collect_sql=False):
        if not all([self.model_name, self.old_fk, self.new_fk, self.new_field]):
            raise ImproperlyConfigured(
                "Please fill out all values for: model_name, old_fk, new_fk, new_field. Read the readme for more explanation!"
            )

        model_identifier = (self.app_label, self.model_name.lower())
        model_state = project_state.models.get(model_identifier)

        new_state_fields = model_state.fields
        new_state_fields[self.new_fk] = self.new_field

        if not self.new_bases:
            self.new_bases = model_state.bases

        self.operations = [
            migrations.RunPython(disable_check_constraints, migrations.RunPython.noop),
            migrations.SeparateDatabaseAndState(
                state_operations=[
                    migrations.DeleteModel(self.model_name),
                    migrations.CreateModel(
                        name=self.model_name,
                        fields=new_state_fields.items(),
                        options=model_state.options,
                        bases=self.new_bases,
                    ),
                ],
                database_operations=[
                    RenamePrimaryKey(
                        model_name=self.model_name.lower(),
                        old_name=self.old_fk,
                        new_name=self.new_fk,
                    ),
                ],
            ),
        ]

        return super().apply(project_state, schema_editor, collect_sql)



import uuid as uuid_module

from django.utils.functional import SimpleLazyObject
from rr.blocks.registry import registered_blocks


def uuid():
    return str(uuid_module.uuid4())


def registered_block_types():
    # pylint: disable=not-an-iterable
    return [block[0] for block in registered_blocks()]


REGISTERED_BLOCK_TYPES = SimpleLazyObject(registered_block_types)



import re

from rr.blocks.main import GridChoiceBlock

from .map import convert_blocks

VALID_GRID_CHOICES = [g[0] for g in GridChoiceBlock.choices]

OFFSET_SANITISATION_REGEX = re.compile(r"col-(?P<size>.*)-offset-(?P<cols>\d)")
COL_SANITISATION_REGEX = re.compile(r"col-(:?xs|sm)")


def offset_pattern_replace(grid):
    return "offset-%(size)s-%(cols)s" % grid.groupdict()


def sanitise_grid(grid):
    grid_classes = []
    block_classes = []
    for value in grid.strip().split(" "):
        value = OFFSET_SANITISATION_REGEX.sub(offset_pattern_replace, value)
        value = COL_SANITISATION_REGEX.sub("col", value)
        if not value:
            continue
        if value in VALID_GRID_CHOICES:
            grid_classes.append(value)
        elif "offset" in value:
            block_classes.append(value)
        else:
            raise ValueError("wat is deze '%s'" % value)

    return " ".join(grid_classes), " ".join(block_classes)


def convert_item_value(obj):
    grid, block_classes = sanitise_grid(obj["grid"])
    obj["grid"] = grid or "col-12"
    if obj["styling"]["block_classes"]:
        obj["styling"]["block_classes"] = "%s %s" % (
            obj["styling"]["block_classes"],
            block_classes,
        )
    else:
        obj["styling"]["block_classes"] = block_classes

    return convert_blocks(obj)



import logging

from django.utils.translation import gettext_lazy as _
from rr.blocks import StylingBlock
from rr.search import get_searchable_content_for_fields

from wagtail import blocks
from wagtail.images.blocks import ImageChooserBlock

logger = logging.getLogger()

HEIGHT_CHOICES = [
    ("px-small", _("Small (Pixel height)")),
    ("px-medium", _("Medium (Pixel height)")),
    ("px-large", _("Large (Pixel height)")),
    ("vh-small", _("Small (Screen height)")),
    ("vh-medium", _("Medium (Screen height)")),
    ("vh-large", _("Large (Screen height)")),
    ("vh-full", _("Full (Screen height)")),
]


class EmptyBlock(blocks.StaticBlock):
    class Meta:
        admin_text = _("The empty block renders nothing on the frontend")
        group = "HTML"
        icon = "wagtail-inverse"
        label = _("Empty")
        template = "roadrunner_bs5/blocks/emptyblock.html"


class ParallaxBlock(blocks.StructBlock):
    image = ImageChooserBlock(label=_("Background"), required=False)
    body = blocks.RichTextBlock(
        label=_("Content"),
        help_text=_("This will be placed on top of the background"),
        required=False,
    )
    height = blocks.ChoiceBlock(
        label=_("Height"),
        help_text=_("The height of the block"),
        choices=HEIGHT_CHOICES,
        default="h-medium",
        required=False,
    )
    horizontal_center = blocks.BooleanBlock(
        label=_("Center horizontally"),
        help_text=_("This will center the text horizontally"),
        required=False,
    )
    overlay = blocks.BooleanBlock(
        label=_("Darken background"),
        help_text=_(
            "This will darken the background behind the text, which can help make text more readable on some images."
        ),
        default=True,
        required=False,
    )
    lazy = blocks.BooleanBlock(
        required=False,
        default=False,
        label=_("Below the fold"),
        help_text=_(
            "If the user needs to scroll before the content becomes visible, it is said to be 'below the fold'. Correctly identifying content as 'Below the fold' will speed up page loading time."
        ),
    )
    styling = StylingBlock()

    def get_searchable_content(self, value):
        return get_searchable_content_for_fields(
            value, self.child_blocks, ["image", "body"]
        )

    class Meta:
        template = "roadrunner_bs5/blocks/parallaxblock.html"
        form_template = "formtemplate/parallax.html"
        preview_template = "preview/bootstrap/parallax.html"
        icon = "image"
        label = _("parallax")
        group = "Bootstrap"


class GoogleMapsBlock(blocks.StructBlock):
    name = blocks.CharBlock(label=_("Name"), help_text=_("Company name"))
    address = blocks.CharBlock(
        label=_("Address"),
        required=False,
        help_text=_("The address to display on the map"),
    )
    aspect_ratio = blocks.ChoiceBlock(
        label=_("Aspect ratio"),
        help_text=_("The shape of the map"),
        choices=[
            ("21by9", "Rectangular"),
            ("1by1", "Square"),
        ],
        default="21by9",
    )
    view = blocks.ChoiceBlock(
        label=_("Display"),
        help_text=_("The type of map detail to display"),
        choices=[
            ("m", _("Map")),
            ("k", _("Satelite")),
            ("h", _("Hybrid")),
            ("p", _("Terrain")),
        ],
        default="m",
    )
    zoom = blocks.ChoiceBlock(
        label=_("Zoom"),
        default="17",
        help_text=_("The map's zoom level"),
        choices=[
            ("0", _("max. zoom out")),
            ("1", _("4.000 km")),
            ("2", _("2.000 km (world)")),
            ("3", _("1.000 km")),
            ("4", _("400 km (continent)")),
            ("5", _("200 km")),
            ("6", _("100 km (country)")),
            ("7", _("50 km")),
            ("8", _("30 km")),
            ("9", _("15 km (area)")),
            ("10", _("8 km")),
            ("11", _("4 km")),
            ("12", _("2 km (city)")),
            ("13", _("1 km")),
            ("14", _("400 m (district)")),
            ("15", _("200 m")),
            ("16", _("100 m")),
            ("17", _("50 m (street)")),
            ("18", _("20 m")),
            ("19", _("10 m")),
            ("20", _("5 m  (house)")),
            ("21", _("2,5 m")),
        ],
    )

    def get_searchable_content(self, value):
        return get_searchable_content_for_fields(
            value, self.child_blocks, ["name", "address"]
        )

    class Meta:
        template = "roadrunner_bs5/blocks/googlemapsblock.html"
        preview_template = "preview/bootstrap/googlemaps.html"
        form_template = "formtemplate/googlemaps.html"
        icon = "ggmaps"
        label = _("maps")
        group = "Extra"


class SlideShowChildBlock(blocks.StructBlock):
    image = ImageChooserBlock(required=False)
    body = blocks.RichTextBlock(required=False)
    position = blocks.ChoiceBlock(
        choices=[
            ("top-left", _("Top left")),
            ("top-right", _("Top right")),
            ("bottom-left", _("Bottom left")),
            ("bottom-right", _("Bottom right")),
            ("middle-left", _("Middle left")),
            ("middle", _("Middle")),
            ("middle-right", _("Middle right")),
        ],
        required=False,
        default="middle-left",
    )

    def get_searchable_content(self, value):
        return get_searchable_content_for_fields(
            value, self.child_blocks, ["image", "body"]
        )

    class Meta:
        label = _("Slide")
        icon = "help"
        group = "Extra"
        preview = ["image"]


class SlideShowBlock(blocks.StructBlock):
    slides = blocks.ListBlock(SlideShowChildBlock())

    # options
    items = blocks.IntegerBlock(
        help_text=_("The number of items you want to see on the screen."), default=1
    )
    slideby = blocks.CharBlock(
        help_text=_("Number of slides going on one 'click'."),
        required=False,
        default="1",
    )
    controls = blocks.BooleanBlock(
        help_text=_("Show next/prev buttons."), required=False, default=True
    )
    autoplaytimeout = blocks.CharBlock(
        label=_("Autoplay timeout (in milliseconds)"),
        help_text=_(
            "Autoplay interval timeout. The time between slides. The amount is in milliseconds"
        ),
        required=False,
    )
    loop = blocks.BooleanBlock(
        help_text=_(
            "Infinite loop. Duplicate last and first items to get loop illusion."
        ),
        required=False,
        default=True,
    )
    nav = blocks.BooleanBlock(help_text=_("Show navigation dots."), required=False)
    autoheight = blocks.BooleanBlock(
        help_text=_("Automatically adjust height based on image height."),
        required=False,
        default=True,
    )
    autoplay = blocks.BooleanBlock(
        help_text=_("Automatically play slides"), required=False
    )
    fixed_height = blocks.ChoiceBlock(
        label=_("Fixed height of all images"),
        help_text=_(
            "This will make it possible to set a height for all the images in the slider. You can put this in pixels or viewheight."
        ),
        choices=HEIGHT_CHOICES,
        required=False,
    )
    lazy = blocks.BooleanBlock(
        required=False,
        default=False,
        label=_("Below the fold"),
        help_text=_(
            "If the user needs to scroll before the content becomes visible, it is said to be 'below the fold'. Correctly identifying content as 'Below the fold' will speed up page loading time."
        ),
    )

    styling = StylingBlock()

    def get_searchable_content(self, value):
        return get_searchable_content_for_fields(value, self.child_blocks, ["slides"])

    class Meta:
        label = _("Slideshow")
        icon = "slider"
        group = "Extra"
        template = "roadrunner_bs5/blocks/slideshow.html"
        form_template = "formtemplate/slideshow.html"
        preview = ["slides"]


class SectionFAQItemBlock(blocks.StructBlock):
    question = blocks.CharBlock(label="Vraag")
    answer = blocks.RichTextBlock(label="Antwoord")

    class Meta:
        label = "FAQ Item"


class SectionItemBlock(blocks.StructBlock):
    label = blocks.CharBlock(label="Label", required=False)
    items = blocks.ListBlock(SectionFAQItemBlock(), label="Items")


class SectionedFAQBlock(blocks.StructBlock):
    header = blocks.RichTextBlock(label="Header heading", required=False)
    text = blocks.TextBlock(label="Header tekst", required=False)

    items = blocks.ListBlock(SectionItemBlock(), label="Items")

    footer_heading = blocks.CharBlock(label="Footer heading", required=False)
    footer_text = blocks.TextBlock(label="Footer tekst", required=False)

    class Meta:
        label = "FAQ"
        template = "roadrunner_bs5/blocks/faqblock.html"
        icon = "openquote"
        group = "Extra"



# Generated by Django 4.2.11 on 2024-03-27 13:56

from django.db import migrations
import rr.fields


class Migration(migrations.Migration):

    dependencies = [
        ("roadrunner_bs5", "0003_rename_accordion_block"),
    ]

    operations = [
        migrations.AlterField(
            model_name="roadrunnerpage",
            name="content",
            field=rr.fields.RoadRunnerField(blank=True, null=True, use_json_field=True),
        ),
    ]



from django.db import models
from django.utils.translation import gettext_lazy as _
from rr.fields import RoadRunnerField

from wagtail.admin.panels import FieldPanel
from wagtail.search import index

from ocyan.plugin.wagtail.models import AbstractBasePage


class RoadrunnerPage(AbstractBasePage):
    template = "roadrunner/roadrunner_page.html"
    content = RoadRunnerField(null=True, blank=True, use_json_field=True)
    noindex = models.BooleanField(default=False)

    content_panels = AbstractBasePage.content_panels + [
        FieldPanel("content", classname="full", disable_comments=True)
    ]

    promote_panels = AbstractBasePage.promote_panels + [FieldPanel("noindex")]

    search_fields = AbstractBasePage.search_fields + [
        index.SearchField("content", partial_match=True),
    ]

    page_description = _("Gives full control over the layout of your page.")

    class Meta:
        verbose_name = _("Pagemaker page")
        verbose_name_plural = _("Pagemaker pages")



from rr.utils import fix_json, fix_richtext

from ocyan.main.loading import get_ocyan_class

from .utils import uuid

convert_item_value = get_ocyan_class(
    "roadrunner_bs5.convert.item", "convert_item_value"
)


def bs4_to_bs5(
    json_content,
    image_keys=(
        "image",
        "product",
        "category",
        "template",
    ),
    richtext_keys=(
        "block_text",
        "tab_content",
        "html",
        "caption",
        "card_content",
        "modal_content",
        "panel_content",
        "media_content",
        "well_content",
        "body",
        "content",
        "text",
        "paragraph",
    ),
):
    """
    Wrap the bs4 rows in a row object.
    """
    result = []
    for container in json_content:
        if container["type"] in ["fixed_width", "full_width"]:
            rows = container["value"]
            container["value"] = {
                "row": [
                    {"type": "item", "value": convert_item_value(kv), "id": uuid()}
                    for kv in rows
                ],
                "styling": {
                    "margin": "",
                    "padding": "",
                    "border": "",
                    "border_radius": "",
                    "background_color": "",
                    "color": "",
                    "font_size": None,
                    "text_align": "",
                    "block_classes": "",
                },
                "id": uuid(),
            }
        bs5_container_with_real_richtext = fix_richtext(
            container, richtext_keys=richtext_keys
        )
        bs5_container_with_correct_image_ids = fix_json(
            bs5_container_with_real_richtext, image_keys=image_keys
        )
        result.append(bs5_container_with_correct_image_ids)

    return result



try:
    from ocyan.plugin.wagtail_oscar_integration.blocks import (
        CategoryChooserBlock,
        ProductChooserBlock,
    )
except ImportError:
    # this is a barebone using roadrunner bs5, there is no wagtail_oscar_integration
    # so there is nothing to register. I would rather do it in the extension's
    # __init__.py to keep everything together and make sure this code is only loaded
    # when the extension is activated but there is nu such luck unfortunately.
    # It will trigger a cyclic import error :(
    pass
else:
    from rr.telepath import register

    from ocyan.plugin.roadrunner_bs5.bs5_wagtail_oscar_integrations_extension.adapters import (
        PreviewAdminChooserAdapter,
    )

    register(PreviewAdminChooserAdapter(), ProductChooserBlock)
    register(PreviewAdminChooserAdapter(), CategoryChooserBlock)



import json
from collections import defaultdict

from ocyan.core.utils import identity
from ocyan.main.loading import get_ocyan_class, get_ocyan_classes

from .utils import REGISTERED_BLOCK_TYPES

# pylint: disable=unbalanced-tuple-unpacking
(
    convert_html,
    convert_carousel,
    convert_iconslider,
    convert_product,
    convert_googlemaps,
    convert_vat,
    convert_embed,
) = get_ocyan_classes(
    "roadrunner_bs5.convert.blocks",
    [
        "convert_html",
        "convert_carousel",
        "convert_iconslider",
        "convert_product",
        "convert_googlemaps",
        "convert_vat",
        "convert_embed",
    ],
)

convert_snippet = get_ocyan_class("roadrunner_bs5.convert.snippet", "convert_snippet")


class BlockMap:
    default_blockmap = {
        "html": convert_html,
        "carousel": convert_carousel,
        "IconSlider": convert_iconslider,
        "HighlightBlock": identity,
        "product": convert_product,
        "googlemaps": convert_googlemaps,
        "vatentry": convert_vat,
        "embed": convert_embed,
    }

    def __init__(self):
        self.block_map = defaultdict(lambda: convert_snippet, self.default_blockmap)

    def register(self, typ, mapper):
        self.block_map[typ] = mapper

    def has_replacement(self, typ):
        return typ in self.block_map

    def replacement(self, typ, blockdef):
        try:
            return self.block_map[typ](blockdef)
        except KeyError as e:
            raise ValueError(
                "Unknown block %s, did you register your custom blocks with rr?\n"
                "See https://git.mandelblog.com/mandel-plugins/ocyan.plugin.roadrunner_bs5/src/branch/master/README.rst\n"
                "Available blocks: %s\n"
                "Blockdef: %s"
                % (
                    typ,
                    json.dumps(list(REGISTERED_BLOCK_TYPES)),
                    json.dumps(blockdef, indent=2),
                )
            ) from e


blockmap = BlockMap()


def convert_blocks(obj):
    if isinstance(obj, list):
        return [convert_blocks(o) for o in obj]
    elif isinstance(obj, dict):
        if "type" in obj:
            if obj["type"] not in REGISTERED_BLOCK_TYPES or blockmap.has_replacement(
                obj["type"]
            ):
                return blockmap.replacement(obj["type"], obj)

        return {key: convert_blocks(value) for key, value in obj.items()}

    return obj



# Generated by Django 3.2.18 on 2023-04-19 14:52

from django.db import migrations
from wagtail.fields import StreamField

import json

def fix_accordion(obj):
    if isinstance(obj, list):
        for child in obj:
            fix_accordion(child)
    elif isinstance(obj, dict):
        for key, value in obj.items():
            if key == "type" and value == "Accordion":
                obj[key] = "accordion"
            fix_accordion(value)

def forward(apps, schema_editor):
    Page = apps.get_model('wagtailcore', 'Page')
    try:
        for page in Page.objects.all():
            content_type = page.content_type
            page_model_class = apps.get_model(content_type.app_label, content_type.model)
            specific_page = page_model_class.objects.get(id=page.id)

            for field in specific_page._meta.get_fields():
                if isinstance(field, StreamField):
                    raw_data = list(getattr(specific_page, field.name).raw_data)
                    fix_accordion(raw_data)
                    setattr(specific_page, field.name, json.dumps(raw_data))
                    specific_page.save()
    except Exception:
        pass

class Migration(migrations.Migration):

    dependencies = [
        ('roadrunner_bs5', '0002_roadrunnerpage_noindex'),
    ]

    operations = [
        migrations.RunPython(forward, migrations.RunPython.noop),
    ]
